// LeafCheck v8 — modes réduits (MESURE, LOUNGE, SPOT, WAVE)

#include <Adafruit_NeoPixel.h>

#define LED_PIN 6
#define NUM_LEDS 61
#define MQ135_PIN A0
#define BUTTON_PIN 2

Adafruit_NeoPixel strip(NUM_LEDS, LED_PIN, NEO_GRB + NEO_KHZ800);

const int airMin = 100;
const int airMax = 600;

const uint8_t brightnessSteps[] = {26, 77, 128, 192, 255};
uint8_t brightnessIndex = 2;

bool lastButtonState = HIGH;
bool buttonPressed = false;
bool longPressTriggered = false;
unsigned long buttonPressTime = 0;
const unsigned long longPressDuration = 800;

enum Mode { MESURE, LOUNGE, SPOT, WAVE };
Mode currentMode = MESURE;

uint16_t loungeHue = 0;
int spotPosition = 0;
const int spotSize = 8;
uint16_t spotHue = 0;

float pulseValue = 0;
bool pulseUp = true;

void setup() {
  delay(1500);
  Serial.begin(9600);
  delay(1000);
  pinMode(BUTTON_PIN, INPUT_PULLUP);

  strip.begin();
  strip.setBrightness(brightnessSteps[brightnessIndex]);
  strip.show();

  for (int i = 0; i < NUM_LEDS; i++) {
    uint16_t hue = map(i, 0, NUM_LEDS - 1, 0, 65535);
    strip.setPixelColor(i, strip.gamma32(strip.ColorHSV(hue)));
    strip.show();
    delay(25);
  }

  fadeToAirColor();
}

void loop() {
  handleButton();

  switch (currentMode) {
    case MESURE: updateAirQuality(); break;
    case LOUNGE: animateLounge(); break;
    case SPOT: animateSpot(); break;
    case WAVE: animateWave(); break;
  }

  delay(20);
}

void handleButton() {
  bool currentState = digitalRead(BUTTON_PIN);

  if (currentState == LOW && lastButtonState == HIGH) {
    buttonPressTime = millis();
    buttonPressed = true;
    longPressTriggered = false;
  }

  if (buttonPressed && currentState == LOW) {
    unsigned long pressDuration = millis() - buttonPressTime;
    if (pressDuration >= longPressDuration && !longPressTriggered) {
      currentMode = static_cast<Mode>((currentMode + 1) % 4);
      blinkTwice();
      Serial.print("MODE: ");
      Serial.println(currentMode);
      longPressTriggered = true;
      buttonPressed = false;
    }
  }

  if (currentState == HIGH && lastButtonState == LOW && buttonPressed && !longPressTriggered) {
    brightnessIndex = (brightnessIndex + 1) % (sizeof(brightnessSteps));
    Serial.print("Luminosité : ");
    Serial.println(brightnessSteps[brightnessIndex]);
    buttonPressed = false;
  }

  lastButtonState = currentState;
}

void blinkTwice() {
  for (int i = 0; i < 2; i++) {
    for (int j = 0; j < NUM_LEDS; j++) strip.setPixelColor(j, strip.Color(255, 255, 255));
    strip.setBrightness(brightnessSteps[brightnessIndex]);
    strip.show();
    delay(100);
    for (int j = 0; j < NUM_LEDS; j++) strip.setPixelColor(j, 0);
    strip.show();
    delay(100);
  }
}

void updateAirQuality() {
  static uint8_t lastRed = 0;
  static uint8_t lastGreen = 255;
  int mqValue = analogRead(MQ135_PIN);
  Serial.println(mqValue);

  mqValue = constrain(mqValue, airMin, airMax);
  int pollutionLevel = map(mqValue, airMin, airMax, 0, 255);
  pollutionLevel = constrain(pollutionLevel, 0, 255);

  uint8_t targetRed = pollutionLevel;
  uint8_t targetGreen = 255 - pollutionLevel;

  for (int s = 0; s <= 4; s++) {
    float t = s / 4.0;
    uint8_t r = lastRed + t * (targetRed - lastRed);
    uint8_t g = lastGreen + t * (targetGreen - lastGreen);
    for (int i = 0; i < NUM_LEDS; i++) strip.setPixelColor(i, strip.Color(r, g, 0));
    strip.setBrightness(brightnessSteps[brightnessIndex]);
    strip.show();
    delay(10);
  }

  lastRed = targetRed;
  lastGreen = targetGreen;
}

void fadeToAirColor() {
  int mqValue = analogRead(MQ135_PIN);
  mqValue = constrain(mqValue, airMin, airMax);
  int pollutionLevel = map(mqValue, airMin, airMax, 0, 255);
  uint8_t targetR = pollutionLevel;
  uint8_t targetG = 255 - pollutionLevel;
  for (int fade = 0; fade <= 30; fade++) {
    float t = fade / 30.0;
    for (int i = 0; i < NUM_LEDS; i++) {
      uint32_t fromColor = strip.getPixelColor(i);
      uint8_t r0 = (fromColor >> 16) & 0xFF;
      uint8_t g0 = (fromColor >> 8) & 0xFF;
      uint8_t b0 = fromColor & 0xFF;
      uint8_t r = r0 + t * (targetR - r0);
      uint8_t g = g0 + t * (targetG - g0);
      uint8_t b = b0 + t * (0 - b0);
      strip.setPixelColor(i, strip.Color(r, g, b));
    }
    strip.setBrightness(brightnessSteps[brightnessIndex]);
    strip.show();
    delay(20);
  }
}

void animateLounge() {
  loungeHue += 256;
  for (int i = 0; i < NUM_LEDS; i++) {
    uint16_t hue = loungeHue + i * 500;
    strip.setPixelColor(i, strip.gamma32(strip.ColorHSV(hue)));
  }
  strip.setBrightness(brightnessSteps[brightnessIndex]);
  strip.show();
}

void animateSpot() {
  strip.clear();
  for (int i = 0; i < spotSize; i++) {
    int index = (spotPosition + i) % NUM_LEDS;
    uint16_t hue = (spotHue + i * 1000) % 65535;
    strip.setPixelColor(index, strip.gamma32(strip.ColorHSV(hue)));
  }
  strip.setBrightness(brightnessSteps[brightnessIndex]);
  strip.show();
  spotPosition = (spotPosition + 1) % NUM_LEDS;
  spotHue += 256;
}

void animateWave() {
  static uint16_t waveHue = 0;
  static int waveCounter = 0;
  unsigned long time = millis();

  waveCounter++;
  if (waveCounter >= 10) {
    waveHue += 1;
    waveCounter = 0;
  }

  for (int i = 0; i < NUM_LEDS; i++) {
    float phase = (time / 600.0) + i * 0.2;
    float intensity = (sin(phase) + 1.0) * 0.5;
    uint8_t level = map(intensity * 255, 0, 255, 0, brightnessSteps[brightnessIndex]);
    uint32_t color = strip.gamma32(strip.ColorHSV(waveHue * 256, 255, level));
    strip.setPixelColor(i, color);
  }
  strip.show();
}
